import { Patch } from 'immer';
import { StoreApi } from 'zustand';
import { Config, HistoryManagement, MappedConfig, RouterOptions } from './store.js';
export declare enum HistoryEventType {
    PUSH = 0,
    REPLACE = 1,
    REGISTER = 2
}
export interface GenericObject {
    [key: string]: any;
}
export interface NamespaceValues<ValueState extends object> {
    /** the amount of elements currently subscribed to the namespaces values */
    subscribers: number;
    values: ValueState;
    initialValues: ValueState;
    mappedConfig: MappedConfig;
    config: Config;
    query: Record<string, string>;
    unsubscribe: () => boolean;
}
export type PushStateFunction<T> = (ns: string, valueCreator: (state: T) => void, routerOptions?: RouterOptions) => Promise<unknown>;
export type ReplaceStateFunction<T> = (ns: string, valueCreator: (state: T) => void, routerOptions?: RouterOptions) => Promise<unknown>;
export type InnerNamespace<V extends Record<string, unknown>> = Record<string, NamespaceValues<V>>;
interface RegistryPayload<ValueState> {
    unsubscribe: () => boolean;
    values: ValueState;
    initialValues: ValueState;
}
export interface StoreState<ThisValue extends Record<string, unknown>, Namespaces extends InnerNamespace<ThisValue> = InnerNamespace<ThisValue>> {
    readonly updateFromQuery: (query: string | URLSearchParams) => void;
    readonly batchReplaceState: <Keys extends keyof Namespaces>(ns: Keys[], fn: (...args: Namespaces[Keys]['values'][]) => void, routerOptions?: RouterOptions) => Promise<unknown>;
    readonly batchPushState: <Keys extends keyof Namespaces>(ns: Keys[], fn: (...args: (Namespaces[Keys]['values'] | undefined)[]) => void, routerOptions?: RouterOptions) => Promise<unknown>;
    namespaces: Namespaces;
    readonly pushState: PushStateFunction<Namespaces[keyof Namespaces]['values']>;
    readonly replaceState: ReplaceStateFunction<Namespaces[keyof Namespaces]['values']>;
    /** registers a new namespace */
    readonly register: <N extends keyof Namespaces>(config: Config, mappedConfig: MappedConfig, ns: N, initialValues: Namespaces[N]['values'], query: Record<string, string>, values: Namespaces[N]['values']) => RegistryPayload<Namespaces[N]['values']>;
    /** will delete all namespaces and remove the history listener */
    readonly unregister: () => void;
    readonly resetPush: (ns: string, routerOptions?: RouterOptions) => void;
    readonly resetReplace: (ns: string, routerOptions?: RouterOptions) => void;
    readonly initialQueries: () => Record<string, string>;
}
type NamespaceProducerFunction<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (state: T[keyof T]) => void;
type InnerNamespaceProducerFunction<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (state: T) => T | void;
export type NamespaceProducer<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (stateProducer: NamespaceProducerFunction<V, T>, eventType: HistoryEventType, ns?: keyof T, routerOptions?: RouterOptions) => Promise<unknown>;
export type GenericConverter<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (stateProducer: InnerNamespaceProducerFunction<V, T>, eventType: HistoryEventType, ns?: keyof T, routerOptions?: RouterOptions) => Promise<unknown>;
export type ImmerProducer<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (stateMapper: (changes: Patch[], values: StoreState<V, T>) => StoreState<V, T>, fn: NamespaceProducerFunction<V, T> & InnerNamespaceProducerFunction<V, T>, eventType: HistoryEventType, ns?: keyof T) => void;
export declare type StateCreator<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (set: NamespaceProducer<V, T> & GenericConverter<V, T>, get: StoreApi<StoreState<V, T>>['getState'], api: StoreApi<StoreState<V, T>>) => StoreState<V, T>;
export declare const historyManagement: <V extends Record<string, unknown>, T extends InnerNamespace<V>>(historyInstance: HistoryManagement) => (apply: StateCreator<V, T>) => (set: ImmerProducer<V, T>, get: () => StoreState<V, T>, api: StoreApi<StoreState<V, T>>) => StoreState<V, T>;
export type ImmerStateCreator<V extends Record<string, unknown>, T extends InnerNamespace<V>> = (fn: ImmerProducer<V, T>, get: StoreApi<StoreState<V, T>>['getState'], api: StoreApi<StoreState<V, T>>) => StoreState<V, T>;
export type SetImmerState<T> = (stateProducer: (state: T) => T, debugMiddlewareKey: string) => void;
export declare const immerWithPatches: <V extends Record<string, unknown>, T extends InnerNamespace<V>>(config: ImmerStateCreator<V, T>) => (set: SetImmerState<StoreState<V, T>>, get: () => StoreState<V, T>, api: StoreApi<StoreState<V, T>>) => StoreState<V, T>;
export declare const converter: <V extends Record<string, unknown>, T extends InnerNamespace<V>>(historyInstance: HistoryManagement) => (set: NamespaceProducer<V, T> & GenericConverter<V, T>, get: () => StoreState<V, T>) => StoreState<V, T>;
export {};
